import QtQuick 1.1
import QtWebKit 1.0

Item {
    id: root

    property string url: defaultURL
    property bool keyingURL: false
    property string defaultURL: application.homeUrl
    property bool noSwipe: true
    property int currentContentWidth:0

    focus: true

    Connections {
        target: application

        onCommandReceived: {
            processCommand(cmd)
        }
    }

    function processCommand(cmd) {
        var parts = cmd.split(" ")
        if (parts.length < 2) {
            console.log("invalid command", cmd)
            return
        }

        switch (parts[1]) {
            case "back":
                webView.back.trigger()
                break
            case "forward":
                webView.forward.trigger()
                break
            case "input":
                break
            case "url":
                var url = parts.slice(2).join(" ") // rejoin parts
                root.url = url
                break
            case "clearCookies":
                console.log("TODO clear cookies")
                break
            case "hidden":
                webView.renderingEnabled = false
                break
            case "shown":
                webView.renderingEnabled = true
                break
            default:
                console.log("unknown command", parts[1])
        }
    }

    function clearCookies() {
        nami.clearCookies()
        //webView.reload.trigger()
    }

    onUrlChanged: {
        webView.url = webView.fixUrl(url)
    }

    BrowserTerms {
        id: webTermsPrompt

        Component.onCompleted: hide()
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.horizontalCenterOffset: 20
        anchors.verticalCenter: parent.verticalCenter

        onYesClicked: {
            hide()
        }

        onNoClicked: {
        }
    }

    Rectangle {
        id: buttonBar

        color: "darkgray"
        radius: 5
        anchors { top: parent.top ; left: parent.left ; right: parent.right }
        height: 40

        Row {
            id: buttonRow
            spacing: 15

            ImageButton {
                id: backButton

                upimage: "graphics/web_leftarrow.png"
                downimage: "graphics/web_leftarrow_DOWN.png"
                enabled: webView.back.enabled
                anchors { top: parent.top ; topMargin: 2}

                onClicked: webView.back.trigger()
            }

            ImageButton {
                id: forwardButton

                upimage: "graphics/web_rightarrow.png"
                downimage: "graphics/web_rightarrow_DOWN.png"
                enabled: webView.forward.enabled
                anchors { top: parent.top ; topMargin: 2 }

                onClicked: webView.forward.trigger()
            }

            ImageButton {
                id: fullscreenButton

                visible: true
                upimage: "graphics/web_fullscreen.png"
                downimage: "graphics/web_fullscreen_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }
                onClicked: {
                    console.log("CMD fullscreen\n")
                }
            }

            ImageButton {
                id: keyboardButton

                upimage: "graphics/web_keyboard.png"
                downimage: "graphics/web_keyboard_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }

                onClicked: {
                    if (keypadManager.checkVisible("inputKeypad")) {
                        keypadManager.hideKeypad("inputKeypad")
                    } else {
                        keypadManager.hideKeypad("urlKeypad")
                        keypadManager.showKeypad("inputKeypad")
                    }
                }
            }

            ImageButton {
                id: homeButton

                visible: application.homeUrl
                upimage: "graphics/web_home.png"
                downimage: "graphics/web_home_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }
                onClicked: root.url = root.defaultURL
            }

            ImageButton {
                id: linksButton

                visible: application.linksUrl
                upimage: "graphics/web_bookmarks.png"
                downimage: "graphics/web_bookmarks_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }
                onClicked: root.url = application.linksUrl
            }

        }
        Rectangle {
            id: urlBar

            color: "white"
            radius: 5
            height: 32
            anchors { left: buttonRow.right ; leftMargin: 15 ;  right: parent.right ; rightMargin: 8 ; top: parent.top; topMargin: 3 }

            Rectangle {
                id: progressBar

                color: "lightblue"
                opacity: 0.6
                radius: 5
                width: (webView.progress < 1) ? webView.progress* urlBar.width : 0
                anchors { left: urlBar.left ; top: urlBar.top ; bottom: urlBar.bottom }
            }

            Flickable {
                id: textFlickable
                anchors { verticalCenter: parent.verticalCenter ; left: parent.left ; leftMargin: 5 }
                clip: true
                width: parent.width - 60
                height: urlEntry.height
                flickableDirection: Flickable.HorizontalFlick
                boundsBehavior: Flickable.DragOverBounds
                interactive: contentWidth - 4 > width
                contentWidth: urlEntry.width+2
                contentHeight: urlEntry.height

                TextInput {
                    id: urlEntry

                    function adjustTextPosition() {
                        var offset = -1
                        var rect = urlEntry.positionToRectangle(cursorPosition)
                        if (rect.x - textFlickable.contentX + 30 > textFlickable.width) {
                            offset = Math.max(0, rect.x - textFlickable.width)
                            // show some trailing characters if not at the end
                            if (cursorPosition < urlEntry.text.length - 1)
                                offset += 30
                        }
                        if (offset >= 0)
                            textFlickable.contentX = offset
                    }

                    width: Math.max(textFlickable.width, implicitWidth)
//                    clip: true
                    Connections {
                        target: webView
                        onUrlChanged: {
                            urlEntry.text = webView.url
                            urlEntry.cursorPosition = 0
                            textFlickable.contentX = 0
                        }
                    }
                    font.pointSize: 12
                    onActiveFocusChanged: {
                        if (activeFocus) {
                            keypadManager.hideKeypad("inputKeypad")
                            keypadManager.showKeypad("urlKeypad")
                        } else {
                            keypadManager.hideKeypad("urlKeypad")
                            urlEntry.cursorPosition = 0
                            textFlickable.contentX = 0
                        }
                    }

                    onTextChanged: {
                        urlEntry.adjustTextPosition()

                    }

                    onAccepted: {
                        root.url = urlEntry.text
                        webView.focus = true
                    }

                    //                MouseArea {
                    //                    anchors.fill: parent
                    //                    onClicked: {
                    //                        urlEntry.forceActiveFocus()
                    //                    }
                    //                }
                }
            }

            ImageButton {
                upimage: "graphics/KeyPad_X_Large.png"
                downimage: upimage
                onClicked: {
                    urlEntry.text = ""
                    urlEntry.focus = true
                    textFlickable.contentX = 0
                }
                anchors { right: parent.right; rightMargin: 0; verticalCenter: parent.verticalCenter }
            }
        }
    }

    Flickable {
        id: flickable

        anchors { top: buttonBar.bottom ; bottom: parent.bottom ; left: parent.left ; right: parent.right }

        contentWidth: Math.max(1000, webView.width)
        contentHeight: Math.max(366, webView.height)
        smooth: false
        clip: true

        WebView {
            id: webView

            focus: true

            settings.pluginsEnabled: true
            settings.javascriptCanOpenWindows: false
            settings.developerExtrasEnabled: true
            settings.javascriptEnabled: root.url.indexOf("facebook.com") < 0
            newWindowParent: webView
            newWindowComponent: Component {
                Item {
                    id: newPageComponent
                    WebView {
                        id: newWindow
                        onUrlChanged: {
                            console.log("Opening", url, "in current window rather than new window")
                            root.url = url

                            var newObject = Qt.createQmlObject("import QtQuick 1.0; Item {}", webView)
                            newPageComponent.parent = newObject
                            newObject.destroy()
                        }
                    }
                }
            }

            url: root.defaultURL

//            onWidthChanged: console.log("webview width",width)
//            onHeightChanged: console.log("webview height",height)

            function fixUrl(url) {
                if(url=="") return url
                if(url[0] == "/") return "file://" + url
                if(url.indexOf(":") < 0) {
                    if(url.indexOf(".") < 0 || url.indexOf(" ") >= 0){
                        print("url fallback ")
                        if(root.defaultURL)
                            return root.defaultURL
                        else
                            return "http://www.truefitness.com"
                    }
                    else
                        return urlRewrite("http://" + url)
                }
                else {
                    url = urlRewrite(url)
                    return url
                }

            }

            function urlRewrite(url) {
                var isFacebook = new RegExp(".*(www\.)?facebook\.com/?$")
                if (isFacebook.test(url) && url.indexOf("m.facebook.com") === -1) {
                    return "https://m.facebook.com"
                } else {
                    return url
                }
            }

            function doZoom(zoom, centerX, centerY) {
                console.log("doZoom called",zoom,centerX,centerY)
                if(centerX) {
                    var sc = zoom* contentsScale
                    scaleAnim.to = sc
                    flickVX.from = flickable.contentX
                    flickVX.to = Math.max(0, Math.min(centerX - flickable.width/2, webView.width*sc-flickable.width))
                    finalX.value = flickVX.to
                    flickVY.from = flickable.contentY
                    flickVY.to = Math.max(0, Math.min(centerY-flickable.height/2, webView.height*sc-flickable.height))
                    finalY.value = flickVY.to
                    quickZoom.start()
                }
            }

            onLoadFailed: {
                if (webView.progress < 1.0) {
                    console.log("Failed to load webpage", root.url, webView.progress)
                    loadFailedMessage.show(3)
                }
            }

            onDoubleClick: {
                if(!heuristicZoom(clickX, clickY, 2.5)) {
                    var zf = flickable.width / contentsSize.width
                    if( zf >= contentsScale)
                        zf = 2.0*contentsScale
                    doZoom(zf, clickX*zf, clickY*zf)
                }else console.log("doing heuristicZoom");
            }
            contentsScale: 1
            onContentsSizeChanged: {
                if(Math.abs(contentsSize.width - currentContentWidth) > 20 ){
                    contentsScale = Math.min(1, flickable.width / (contentsSize.width / contentsScale))
                    currentContentWidth = contentsSize.width
                    console.log("onContentsSizeChanged", contentsScale, flickable.width, contentsSize.width, currentContentWidth, progress)
                }
            }
            onZoomTo: doZoom(zoom, centerX, centerY)
            onUrlChanged: {
                flickable.contentX = 0
                flickable.contentY = 0
                if(url != root.url)
                    root.url = url
            }

            transformOrigin: Item.TopLeft
            preferredWidth: root.width
            preferredHeight: root.height - 40
            anchors.bottom: parent.bottom
            anchors.horizontalCenter: parent.horizontalCenter
            smooth: false
        }
    }

    GrowlMessage {
        id: loadFailedMessage
        message: '<img src="graphics/red_x.png" /> ' + qsTr("The Requested Page Could Not Be Loaded") //+ application.emptyString
        anchors { top: buttonBar.bottom }
    }

//    ImageButton { // TODO add ImageButton
//        id: topButton

//        upimage: "graphics/web_leftarrow.png"
//        downimage: "graphics/web_leftarrow_DOWN.png"
//        visible: flickable.contentY > 25
//        opacity: 0.5
//        rotation: 90
//        anchors { top: flickable.top  ; right: flickable.right }

//        onClicked: scrollToTop.start()

//        NumberAnimation {
//            id: scrollToTop
//            target: flickable
//            property: "contentY"
//            easing.type: Easing.Linear
//            duration: 500
//            to: 0
//        }
//    }

    Rectangle {
        id: scrollPos

        color: "gray"
        opacity: 0.5
        width: 10
        height: flickable.height * ( flickable.height / webView.contentsSize.height)

        anchors { right: flickable.right ; top: flickable.top ; topMargin: flickable.contentY * ( flickable.height / flickable.contentHeight)}
    }

    SequentialAnimation {
        id: quickZoom

        PropertyAction { target: webView ; property: "renderingEnabled" ; value: false }
        ParallelAnimation {
            NumberAnimation {
                id: scaleAnim
                target: webView
                property: "contentsScale"
                easing.type: Easing.Linear
                duration: 200
            }
            NumberAnimation {
                id: flickVX
                target: flickable
                property: "contentX"
                easing.type: Easing.Linear
                duration: 200
                from: 0
                to: 0
            }
            NumberAnimation {
                id: flickVY
                target: flickable
                property: "contentY"
                easing.type: Easing.Linear
                duration: 200
                from: 0
                to: 0
            }
        } // parallel animation
        PropertyAction { id: finalX ; target: flickable ; property: "contentX" ; value: 0 }
        PropertyAction { id: finalY ; target: flickable ; property: "contentX" ; value: 0 }
        PropertyAction { target: webView ; property: "renderingEnabled" ; value: true }
    }
}
